Index: main/slinfactory.c
===================================================================
--- main/slinfactory.c	(revision 113008)
+++ main/slinfactory.c	(working copy)
@@ -157,3 +157,21 @@
 {
 	return sf->size;
 }
+
+void ast_slinfactory_flush(struct ast_slinfactory *sf)
+{
+        struct ast_frame *fr = NULL;
+
+        if (sf->trans) {
+                ast_translator_free_path(sf->trans);
+                sf->trans = NULL;
+        }
+
+        while ((fr = AST_LIST_REMOVE_HEAD(&sf->queue, frame_list)))
+                ast_frfree(fr);
+
+        sf->size = sf->holdlen = 0;
+        sf->offset = sf->hold;
+
+        return;
+}
Index: include/asterisk/slinfactory.h
===================================================================
--- include/asterisk/slinfactory.h	(revision 113008)
+++ include/asterisk/slinfactory.h	(working copy)
@@ -48,6 +48,7 @@
 int ast_slinfactory_feed(struct ast_slinfactory *sf, struct ast_frame *f);
 int ast_slinfactory_read(struct ast_slinfactory *sf, short *buf, size_t samples);
 unsigned int ast_slinfactory_available(const struct ast_slinfactory *sf);
+void ast_slinfactory_flush(struct ast_slinfactory *sf);
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
Index: main/audiohook.c
===================================================================
--- main/audiohook.c	(revision 113008)
+++ main/audiohook.c	(working copy)
@@ -128,14 +128,25 @@
 int ast_audiohook_write_frame(struct ast_audiohook *audiohook, enum ast_audiohook_direction direction, struct ast_frame *frame)
 {
 	struct ast_slinfactory *factory = (direction == AST_AUDIOHOOK_DIRECTION_READ ? &audiohook->read_factory : &audiohook->write_factory);
-	struct timeval *time = (direction == AST_AUDIOHOOK_DIRECTION_READ ? &audiohook->read_time : &audiohook->write_time);
+	struct timeval *time = (direction == AST_AUDIOHOOK_DIRECTION_READ ? &audiohook->read_time : &audiohook->write_time), previous_time;
 
+	/* Record previous feeding time */
+	previous_time = *time;
+
+	/* Update last fed time */
+	*time = ast_tvnow();
+
+	/* If it has been quite some time since we were last fed flush both factories so they remain in sync */
+	if (ast_tvdiff_ms(*time, previous_time) > 60) {
+		if (option_debug)
+			ast_log(LOG_DEBUG, "Flushing audiohook %p so it remains in sync\n", audiohook);
+		ast_slinfactory_flush(&audiohook->read_factory);
+		ast_slinfactory_flush(&audiohook->write_factory);
+	}
+
 	/* Write frame out to respective factory */
 	ast_slinfactory_feed(factory, frame);
 
-	/* Update last fed time for the above factory */
-	*time = ast_tvnow();
-
 	/* If we need to notify the respective handler of this audiohook, do so */
 	if ((ast_test_flag(audiohook, AST_AUDIOHOOK_TRIGGER_MODE) == AST_AUDIOHOOK_TRIGGER_READ) && (direction == AST_AUDIOHOOK_DIRECTION_READ)) {
 		ast_cond_signal(&audiohook->trigger);
